#!/usr/bin/perl
#
# $Id: parse_slapd_conf.pl,v 1.2 2002/01/22 00:04:57 seb Exp $
# This script parse the slapd.conf for reading and writing
#
#    OpenLDAP Webmin Module
#    Copyright (C) 2000 by S. Namche <snameche@virtual-net.fr>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#

use Text::ParseWords;

$GLOBAL_TYPE  = '_GLOBAL_';  # Type of the backend that contains global options
%conf_options = ();          # Hash of hashes of function references for options of each backend type

my @backends;  # List of all backends
my $cbackend;  # Number of the current backend
my $mode;      # What parse_* functions should do: 0=>read, 1=>write, 2=>append


# Compares 2 lists, order is not important if third param is true
sub cmp_lists {
    my @one  = @{$_[0]};
    my @two  = @{$_[1]};
    my $sort = $_[2] || 0;

    return 0 if scalar(@one) != scalar(@two);
    @one = sort @one if $sort;
    @two = sort @two if $sort;
    for my $i (0..$#one) {
	return 0 if $one[$i] ne $two[$i];
    }
    return 1;
}

# Compares 2 hashes
sub cmp_hashes {
    my %one = %{$_[0]};
    my %two = %{$_[1]};

    return 0 if scalar(keys %one) != scalar(keys %two);
    foreach( keys %one ) {
	return 0 if $one{$_} ne $two{$_};
    }
    return 1;
}

# Switch from one backend to another
sub change_backend {
    my $cbackend_type = shift;

    if( $mode == 0 ) {
	push @backends, { type => $cbackend_type };
	$cbackend = $#backends;
    }
    elsif( $mode == 1 ) {
	if( $cbackend > -1 ){
	    $mode = 2;
	    foreach( keys %{$conf_options{$backends[$cbackend]{type}}} ){
		&{$conf_options{$backends[$cbackend]{type}}{$_}}($_);
	    }
	    $mode = 1;
	}
	$cbackend++;
    }
}

sub parse_attribute {
    my $option = shift;

    if( $mode == 2 ){
	foreach my $attr ( keys %{$backends[$cbackend]{$option}} ){
	my @tabl =  @{$backends[$cbackend]{$option}{$attr}};
        foreach (@tabl) {
 s/^DESC/\n\tDESC/;
s/^SYNTAX/\n \tSYNTAX/;
s/^SUP/\n \tSUP/;
s/^SINGLE/\n \tSINGLE/;
s/^COLLEC/\n \tCOLLEC/;
s/^OBSOL/\n \tOBSOL/;
s/^USAGE/\n \tUSAGE/;
s/^NO-U/\n \tNO-U/;
s/^DESC/\n \tDESC/;
s/^EQUAL/\n \tEQUAL/;
s/^ORDE/\n \tORDE/;
s/^SUBS/\n \tSUBS/;

}     

 #error ("je passe $mode @tabl");
   
    print NEW "\n$option  ", join( ' ', @tabl ), "\n\n";
	}
	return;
    }

    my $attr = shift;
    my @attrt = @_;
$ligne = join ('#',@attrt);   
$attr = &extract_attribute ('NAME' ,@attrt); 
    if( $mode == 0 ){
 	$backends[$cbackend]{$option} = {} unless defined $backends[$cbackend]{$option};
	$backends[$cbackend]{$option}{$attr} = \@attrt;
    }
    elsif( $mode == 1 ){
#    error ("je passe $mode");
    return 1 unless defined $backends[$cbackend]{$option}{$attr};
	my @new_attr = @{$backends[$cbackend]{$option}{$attr}};
	delete $backends[$cbackend]{$option}{$attr};
	if( ! cmp_lists( \@new_attr, \@attrt, 1 )){
	    print NEW  join(' ',@new_attr) , "\n";
	    return 1;
	}
    }
}

sub parse_access {
    my $option = shift;

    if( $mode == 2 ){
	foreach( @{$backends[$cbackend]{$option}} ){
	    my @new_acl = @$_;
	    print NEW "\naccess to $new_acl[0] $new_acl[1]\n";
 	    for( my $i=2; $i<$#new_acl; $i+=2 ){
		print NEW "\tby $new_acl[$i] $new_acl[$i+1]\n";
	    }
	}
	return;
    }

    print STDERR "parse_access: erreur de syntaxe, 'to' attendu\n"  unless shift eq 'to';
    my @acl;
    $acl[0] = shift;
    $acl[1] = shift;
    if( $acl[0] =~ /^attr(ibute)?=/ ){
	$acl[1] = $acl[0];
	$acl[0] = '';
	unshift @_, 'by';
    }
    elsif( $acl[1] eq 'by' ){
	$acl[1] = '';
	unshift @_, 'by';
    }
    while( @_ ) {
	print STDERR "parse_access: erreur de syntaxe, 'by' attendu\n"  unless shift eq 'by';
	push @acl, shift;
	push @acl, shift;
    }

    if( $mode == 0 ){
	$backends[$cbackend]{$option} = [] unless defined $backends[$cbackend]{$option};
	push @{$backends[$cbackend]{$option}}, \@acl;
    }
    elsif( $mode == 1 ){
	return 1 unless @{$backends[$cbackend]{$option}};
	my @new_acl = @{shift @{$backends[$cbackend]{$option}}};
	if( ! cmp_lists( \@new_acl, \@acl )){
	    print NEW "access to $new_acl[0] $new_acl[1]\n";
 	    for( my $i=2; $i<$#new_acl; $i+=2 ){
		print NEW "\tby $new_acl[$i] $new_acl[$i+1]\n";
	    }
	    return 1;
	}
    }
}

sub parse_defaultaccess {
    my $option = shift;

    if( $mode == 2 ){
	print NEW "$option $backends[$cbackend]{$option}\n" if $backends[$cbackend]{$option};
	return;
    }

    if( $mode == 0 ){
	print STDERR "parse_defaultaccess: invalid access: $_[0]\n"
	    if not $_[0] =~ /^(none|compare|search|read|write|delete)$/;
	$backends[$cbackend]{$option} = $_[0];
    }
    elsif( $mode == 1 ) {
	return 1 unless defined $backends[$cbackend]{$option};
	my $new_value = $backends[$cbackend]{$option};
	delete $backends[$cbackend]{$option};
	if( $new_value ne $_[0] ){
	    print NEW "\n$option $backends[$cbackend]{$option}\n";
	    return 1;
	}
    }
}
sub parse_password {
    my $option = shift;
    $optionil = $option;
    $optionil=~s/_/-/g ;
    if( $mode == 2 ){
	print NEW "$optionil $backends[$cbackend]{$option}\n" if $backends[$cbackend]{$option};
	return;
    }

    if( $mode == 0 ){
#	print STDERR "parse_password: invalid password schema: $_[0]\n"
#	    if not $_[0] =~ /\{\w+\}/;
#	print STDERR "parse_password: invalid password schema: $_[0]\n"
#	    if not $_[0] =~ /\{(SSHA|SHA|MD5|SMD5|CRYPT)\}/;
	$backends[$cbackend]{$option} = $_[0];
    }
    elsif( $mode == 1 ) {
	return 1 unless defined $backends[$cbackend]{$option};
	my $new_value = eval{qq($backends[$cbackend]{$option})};
	delete $backends[$cbackend]{$option};
	if( $new_value ne $_[0] ){
	    print NEW "\n$optionil $new_value\n";
	    return 1;
	}
    }
}

sub parse_include {
    my $option = shift;

    if( $mode == 2) {
	foreach( @{$backends[$cbackend]{$option}} ) {
	    print NEW "$option $_\n";
	}
    }

    if( $mode == 0 ){
	$backends[$cbackend]{$option} = [] unless defined $backends[$cbackend]{$option};
	push @{$backends[$cbackend]{$option}}, $_[0];
    }
    elsif( $mode == 1 ){
	return 1 unless @{$backends[$cbackend]{$option}};
	my $new_value = shift( @{$backends[$cbackend]{$option}} );
	if( $new_value ne $_[0] ){
	    print NEW "$option $new_value\n";
	    return 1;
	}
    }
}

sub parse_index {
    my $option = shift;

    if( $mode == 2 ) {
	foreach( @{$backends[$cbackend]{$option}} ) {
	    print NEW "$option ", join(',', @{$backends[$cbackend]{$option}[0]{attrs}}), " ",
		                , join(',', @{$backends[$cbackend]{$option}[0]{types}}), "\n";
	}
	return;
    }

    my $args = join( " ", @_ );
    $args =~ s/\s*,\s*/,/g;
    my ($attrs, $types) = split( " ", $args );
    my %index;
    $index{attrs} = [ split( /,/, $attrs )];
    $index{types} = [ split( /,/, $types )];

    if( $mode == 0 ){
	$backends[$cbackend]{$option} = [] unless defined $backends[$cbackend]{$option};
	$backends[$cbackend]{index}[scalar @{$backends[$cbackend]{index}}] = \%index;
    }
    elsif( $mode == 1 ){
	return 1 unless @{$backends[$cbackend]{$option}};
	my %new_index = %{ shift( @{$backends[$cbackend]{$option}} )};
	if( !cmp_lists( $new_index{attrs}, $index{attrs})
         || !cmp_lists( $new_index{types}, $index{types}) ){
	    print NEW "$option ", join(',', @{$new_index{attrs}}), " ",
		                , join(',', @{$new_index{types}}), "\n";
	    return 1;
	}
    }
}

sub parse_objectclass {
    my $option = shift;

    if( $mode == 2 ){
	foreach my $oc ( keys %{$backends[$cbackend]{$option}} ){
	    print NEW "\n$option $oc\n";
	    print NEW "\trequires\n\t\t", join(",\n\t\t", @{$backends[$cbackend]{$option}{$oc}{requires}}), "\n"
		if @{$backends[$cbackend]{$option}{$oc}{requires}};
	    print NEW "\tallows\n\t\t", join(",\n\t\t", @{$backends[$cbackend]{$option}{$oc}{allows}}), "\n"
		if @{$backends[$cbackend]{$option}{$oc}{allows}};
            print NEW "\n";
	}
	return;
    }

    my $oc = shift;
    my $section;
    my %attrs = ( requires => [], allows => [] );
    my $args = join( ' ', @_ );
    $args =~ s/\s*,\s*/, /g;
    foreach( split( / /, $args )) {
	s/,\s*$//;
	if( /^(requires|allows)$/ ) {
	    $section = $_;
	    next;
	}
	if( ! $section) {
	  #  print STDERR "parse_objectclass: objectclass $oc: erreur de syntaxe\n";
	  #  next;
	}
	push @{$attrs{$section}}, $_;
    }

    if( $mode == 0 ){
	$backends[$cbackend]{$option} = {} unless defined $backends[$cbackend]{$option};
	$backends[$cbackend]{objectclass}{$oc} = \%attrs;
    }
    elsif( $mode == 1 ){
	return 1 unless defined $backends[$cbackend]{$option}{$oc};
	my %new_oc = %{$backends[$cbackend]{$option}{$oc}};
	delete $backends[$cbackend]{$option}{$oc};
	if( ! cmp_lists( \@{$new_oc{requires}}, \@{$attrs{requires}}, 1) ||
	    ! cmp_lists( \@{$new_oc{allows}}, \@{$attrs{allows}}, 1) ){
	
	    print NEW "$option $oc\n";
	    print NEW "\trequires\n\t\t", join(",\n\t\t", @{$new_oc{requires}}), "\n" if @{$new_oc{requires}};
	    print NEW "\tallows\n\t\t", join(",\n\t\t", @{$new_oc{allows}}), "\n" if @{$new_oc{allows}};
	    return 1;
	}
    }
}

sub parse__file {
    my $option = shift;

    if( $mode == 2 ){
	print NEW "\n$option $backends[$cbackend]{$option}\n" if $backends[$cbackend]{$option};
	return;
    }

    if( $mode == 0 ){
	$backends[$cbackend]{$option} = $_[0];
    }
    elsif( $mode == 1 ){
	return 1 unless defined $backends[$cbackend]{$option};
	my $new_value = $backends[$cbackend]{$option};
	delete $backends[$cbackend]{$option};
	if( $new_value ne $_[0] ){
	    print NEW "$option $new_value\n";
	    return 1;
	}
    }
}

sub parse__string {
    my $option = shift;

    if( $mode == 2 ){
	print NEW "\n$option $backends[$cbackend]{$option}\n" if $backends[$cbackend]{$option};
	return;
    }

    if( $mode == 0 ){
	$backends[$cbackend]{$option} = $_[0];
    }
    elsif( $mode == 1 ){
	return 1 unless defined $backends[$cbackend]{$option};
	my $new_value = $backends[$cbackend]{$option};
	delete $backends[$cbackend]{$option};
	if( $new_value ne $_[0] ){
	    print NEW "$option \"$new_value\"\n";
	    return 1;
	}
    }
}

sub parse__boolean {
    my $option = shift;

    if( $mode == 2 ){
	printf NEW "\n$option %s\n", $backends[$cbackend]{$option} ? 'on' : 'off'
	    if defined $backends[$cbackend]{$option};
	return;
    }

    if( $mode == 0 ){
	$backends[$cbackend]{$option} = int( $_[0] =~ /^on$/i );
    }
    elsif( $mode == 1 ){
	return 1 unless defined $backends[$cbackend]{$option};
	my $new_value = $backends[$cbackend]{$option} ? 'on' : 'off';
	delete $backends[$cbackend]{$option};
	if( $new_value ne $_[0] ){
	    print NEW "$option $new_value\n";
	    return 1;
	}
    }
}

sub parse__integer {
    my $option = shift;

    if( $mode == 2 ){
	print NEW "$option $backends[$cbackend]{$option}\n" if $backends[$cbackend]{$option};
	return;
    }

    if( $mode == 0 ){
	$backends[$cbackend]{$option} = $_[0];
    }
    elsif( $mode == 1 ){
	return 1 unless defined $backends[$cbackend]{$option};
	my $new_value = $backends[$cbackend]{$option};
	delete $backends[$cbackend]{$option};
	if( $new_value != $_[0] ){
	    print NEW "$option $new_value\n";
	    return 1;
	}
    }
}

sub parse_unimplemented {
    return if $mode;
    printf STDERR "parse_unimplemented: Ignoring configuration directive \"%s\" at line %d\n"
	, $_[0], SLAPD->input_line_number();
}


sub init_conf_options {
    my %conf_global = ( 'attributetype' => \&parse_attribute,
			'password-hash' => \&parse_password,
			'access'        => \&parse_access,
			'defaultaccess' => \&parse_defaultaccess,
			'include'       => \&parse_include,
			'pidfile'       => \&parse__file,
			'argsfile'      => \&parse__file,
			'loglevel'      => \&parse__integer,
			'objectclass'   => \&parse_objectclass,
			'referral'      => \&parse__string,
			'schemacheck'   => \&parse__boolean,
			'sizelimit'     => \&parse__integer,
			'srvtab'        => \&parse__file,
			'timelimit'     => \&parse__integer
			);

    my %conf_backend = ( 'lastmod'        => \&parse__boolean,
			 'readonly'       => \&parse__boolean,
			 'replica'        => \&parse_unimplemented,
			 'replogfile'     => \&parse_unimplemented,
			 'rootdn'         => \&parse__string,
			 'rootpw'         => \&parse__string,
			 'suffix'         => \&parse__string,
			 'update_dn'      => \&parse_unimplemented
			 );

    $conf_options{$GLOBAL_TYPE} = { %conf_global };

    $conf_options{ldbm} = { %conf_global,
			    %conf_backend,
			    'cachesize'      => \&parse__integer,
			    'dbcachesize'    => \&parse__integer,
			    'dbcachenowsync' => \&parse_unimplemented,
			    'directory'      => \&parse__string,
			    'index'          => \&parse_index,
			    'mode'           => \&parse__integer
			    };

    $conf_options{shell} = { %conf_global,
			     %conf_backend,
			     'bind'    => \&parse__file,
			     'unbind'  => \&parse__file,
			     'search'  => \&parse__file,
			     'compare' => \&parse__file,
			     'modify'  => \&parse__file,
			     'modrdn'  => \&parse__file,
			     'add'     => \&parse__file,
			     'delete'  => \&parse__file,
			     'abandon' => \&parse__file
			     };

    $conf_options{passwd} = { %conf_global,
			      %conf_backend,
			      'file' => \&parse__file
			      };
}

sub parse_expr {
    my @words = quotewords( '\s+', '"', @_ );
    @words = grep( /.+/, @words );
    @words = map( { s/^"(.*)"$/$1/; $_} @words );
    $words[0] = lc( $words[0] );
    $words[0] =~ s/-/_/g; 
    if( $words[0] =~ /^database$/ ) {
	change_backend( lc( $words[1] ));
	print NEW "$_[0]\n" if $mode == 1;
	return;
    }

    if( $conf_options{$backends[$cbackend]{type}}{$words[0]} ) {
	if( ! &{$conf_options{$backends[$cbackend]{type}}{$words[0]}}(@words) && $mode==1 ){
	    foreach (@_) { print NEW "$_\n" };
	}
    }
    elsif( $mode == 1){
	foreach (@_) { print NEW "$_\n" };
    }
}

sub parse_slapd_conf {
    my $file = shift;

    $mode     = 0;
    @backends = ();
    open SLAPD, $file or return;

    init_conf_options();
    change_backend( $GLOBAL_TYPE );
    my @expr = ();

    while (<SLAPD>) {
	chomp;
	next if /^#/;
	next if /^\s*$/;

	if( /^\s/ ) {
	    push @expr, $_;
	    next;
	}

	parse_expr( @expr ) if @expr;
	@expr = ( $_ );
    }
    parse_expr( @expr ) if @expr;

    close SLAPD;
    return @backends;
}

sub rename_slapd_conf_file {
    my( $current, $new, $backup ) = @_;
    unlink $backup if -f $backup;
    rename $current, $backup;
    rename $new, $current;
}

sub write_slapd_conf {
    my( $file, $new ) = @_;

    $mode = 1;
    @backends = @$new;
    $cbackend = -1;

    open SLAPD, $file or die( "parse_slapd_conf: open: $file: $!" );
    open NEW, ">$file.tmp" or die( "parse_slapd_conf: open: $file.tmp: $!" );

    init_conf_options();
    change_backend( $GLOBAL_TYPE );
    my @expr = ();
    my @comments = ();

    while (<SLAPD>) {
	chomp;

	if( /^#/ | /^\s*$/ ) {
	    push @comments, "$_\n";
	    next;
	}

	if( /^\s/ ) {
	    push @expr, $_;
	    next;
	}

	parse_expr( @expr ) if @expr;
	@expr = ( $_ );
	print NEW @comments;
	@comments = ();
    }
    parse_expr( @expr ) if @expr;
    print NEW @comments;
    change_backend( 'dummy' );

    close NEW;
    close SLAPD;
    rename_slapd_conf_file( $file, "$file.tmp", "$file.back" );
}

sub delete_backend {
    my( $file, $suffix ) = @_;

    open SLAPD, $file or die( "delete_backend: open: $file: $!" );
    open NEW, ">$file.tmp" or die( "delete_backend: open: $file.tmp: $!" );

    my @lines;
    my $write = 1;

    while( my $line = <SLAPD> ){
	if( $line =~ /^database\s/ ){
	    if( $write ){
		foreach( @lines ) { print NEW $_; }
	    }
	    @lines = ();
	    $write = 1;
	}
	if( $line =~ /^suffix\s/ && $line =~ /^suffix\s+"?$suffix"?\s*$/ ){
	    $write = 0;
	}
	push @lines, $line;
    }
    if( $write ){
	foreach( @lines ) { print NEW $_; }
    }

    close NEW;
    close SLAPD;
    rename_slapd_conf_file( $file, "$file.tmp", "$file.back" );
}

sub add_backend {
    my( $file, $type, $suffix ) = @_;

    open SLAPD, $file or die( "add_backend: open: $file: $!" );
    open NEW, ">$file.tmp" or die( "delete_backend: open: $file.tmp: $!" );

    my $b = 1;
    while( <SLAPD> ){
	$b++ if /^database\s/;
	print NEW $_;
    }

    print NEW "\n\ndatabase $type\nsuffix \"$suffix\"\n";

    close NEW;
    close SLAPD;
    rename_slapd_conf_file( $file, "$file.tmp", "$file.back" );
    return $b;
}

1;
