/*
 *    author: Lloyd Dupont
 *    mail: lloyd@galador.net
 *    Copyright (C) 2000 Lloyd Dupont
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * this class provide an easy way to post action to be executed later,
 * in an other thread. <BR>
 * This usefull to coutourn some UI bug with applet or post low-level priority
 * task, as the inner thread is low priority.
 *
 * @see #add
 * @see #dispose
 */
public class BackgroundAction
{
    Thread thread;
	Vector toDo = new Vector();

    public BackgroundAction() {}

    protected void init() {
        if(thread != null)
            dispose();

        thread = new Thread(new Loop());
		thread.setPriority(Thread.MIN_PRIORITY);
        thread.start();
    }
    boolean running() { return thread != null; }

	/**
	 * "terminate" the BackgroundAction object by stopping its looping thread.
	 */
    public void dispose() {
        if(thread != null)
            thread.interrupt();
        thread = null;
    }

	/**
	 * add something to do.
	 */
    public void add(Action a) {
		if(a == null)
			return;
		if(thread == null)
			init();

		synchronized(toDo) {
			toDo.addElement(a);
			toDo.notify();
		}
    }

    class Loop implements Runnable {
        public void run() {
            Thread current = Thread.currentThread();
			Action a = null;
			int size;

            while(!Thread.interrupted()) {
				synchronized(toDo) {
					if(toDo.size() == 0)
						try { toDo.wait(); }
						catch(InterruptedException ie) {}
					else {
						a = (Action) toDo.elementAt(0);
						toDo.removeElementAt(0);
					}
				}
				if(a != null) {
					a.action(null);
					a = null;
				}
            }
        }
    }
}
