/*
 *    author: Lloyd Dupont
 *    mail: lloyd@galador.net
 *    Copyright (C) 2000 Lloyd Dupont
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
import java.applet.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * the "MenuApplet". it takes, as sole parameter (MENU), a PList with all
 * commands. A good idea is to have it on one frame and your page on the other.
 * the source comes with a very tiny example of utilisation. you could also
 * run it as an application with java TestFrame
 *
 * @see MenuPanel
 * @see MenuController
 */
public class MenuApplet extends Applet implements Action, Runnable
{
	MenuPanel menu;
	TextArea waitPanel;

	public MenuApplet() {
		setLayout(new BorderLayout());
		waitPanel = new TextArea("", 20, 20, TextArea.SCROLLBARS_HORIZONTAL_ONLY);
		add(waitPanel, BorderLayout.CENTER);
		waitPanel.setText("loading data...");
		waitPanel.setEditable(false);
	}
	/** for test reason it is also a "do nothing" action */
	public void action(String key) {
		System.out.println("action \"" + key + "\" invoked");
	}
	public void init() {
		if(menu == null)
			new Thread(this).start();
	}
	public void run() {
		if(menu != null)
			return;

		URLBuilder.addURLPath(getDocumentBase());

		String s = getParameter("menu");
		if(s == null) {
			waitPanel.setText("no menu.");
			return;
		}

		Reader r = null;
		try {
			URL url = URLBuilder.find(s);
			r = new InputStreamReader(url.openStream());
			Object m = PList.read(r);
			Object p = null;
			try { p = PList.read(r); }
			catch(Exception ex) {}

			// initialisizing
			menu = new MenuPanel(m);
			if(p != null)
				menu.getController().init((Hashtable) p);
			menu.getController().applet = this;

			setEnabled(true);
			removeAll();
			// add(menu, BorderLayout.CENTER);
			// add(menu)
			doLayout();
			menu.repaint();
		}
		catch(Exception ex) {
			ex.printStackTrace();
			waitPanel.setText("can't open menu " + menu);
			waitPanel.append("\nexception: " + ex);
		}
		if(r != null)
			try { r.close(); }
			catch(IOException ioe) {}
	}
	public void stop() {
		if(menu != null) {
			menu.setSelected(-1);
			menu.getController().dispose();
		}
	}

    public void update(Graphics g) { paint(g); }
    public void paint(Graphics g) {
		if(menu != null)
	    	menu.paint(g);
		else
			waitPanel.paint(g);
	}

	public String getAppletInfo() {
		return "\"MenuApplet\" by Lloyd Dupont (lloyd@galador.net) in 2001\n" +
			"Hope it will well deserve you but, as often, comes with no garanty.";
	}
	public String[][] getParameterInfo() {
		return new String[][] {
			{"name", "URL", "url of the configuration plist for all commands."}
		};
	}
}