/*
 *    author: Lloyd Dupont
 *    mail: lloyd@galador.net
 *    Copyright (C) 2000 Lloyd Dupont
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 * the main menu class. it automatically maintain submenu and such stuff.<BR>
 * All menu configuration is provided through a given Vector of Hastable PList.
 * The following key are relevant (for each command's Hashtable).
 * <UL>
 * <LI> <B>name</B> is a String which would be displayed as the menu label.
 * "--" is a special string to display a separator.
 * <LI> <B>icon</B> is an URL to an image
 * <LI> <B>child</B> is either a Vector plist describing submenu or an URL
 * pointing to a file containing such a PList
 * <LI> <B>href</B> the URL to be displayed
 * <LI> <B>target</B> the target frame of the href. it is usefull to have the
 * JMenu in a fame and update the other one.
 * <LI> <B>javascript</B> a javascript String script command to be executed.
 * to <U>enable javascript for your Applet</U> you should tag it with MAYSCRIPT
 * <LI> <B>applet</B> name of an applet on the same frame (tagged with 'NAME=')
 * which should implements <A HREF=Action.html>Action</A> interface
 * <LI> <B>action</B> used in conjonction with <B>applet</B>. the argument
 * to <TT>applet.action()</TT> method.
 * </UL>
 *
 * @see PList
 * @see MenuController
 *
 * @see #setCommands
 * @see #getController
 */
public class MenuPanel extends Panel
{
    MenuCell components[];
    Insets inset = new Insets(1, 1, 1, 1);
    transient int startText = -1;
    transient int selected = -1;
    transient MenuWindow submenu;
    MenuController controller;
    Vector commands;
    transient Image back_buffer;

	/** build a new MenuPanel */
    public MenuPanel() {
        enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK | AWTEvent.MOUSE_EVENT_MASK);
        enableEvents(AWTEvent.COMPONENT_EVENT_MASK);
        controller = new MenuController(this);
    }
	/** build a new MenuPanel. setting o as Commands */
    public MenuPanel(Object o) {
        this();
        setCommands(o);
    }
    MenuPanel(MenuController mc) {
        this();
        setController(mc);
    }
	/**
	 * return the controller of this menu. it should be used to fix any
	 * specific UI defaults wanted and available. any top menu automatically
	 * create its controller and set it for its child.
	 * @see MenuController
	 */
    public MenuController getController() { return controller; }
    public void setController(MenuController mc) {
        controller = mc;
        if(mc.bgcolor != null)
            setBackground(mc.bgcolor);
        if(mc.textcolor != null)
            setForeground(mc.textcolor);
        if(mc.font != null)
            setFont(mc.font);
    }

    public Dimension getMinimumSize() {
        return new Dimension(50, 50);
    }
    public Dimension getPreferredSize() {
        if(components == null || components.length == 0)
            return new Dimension(0, 0);
        int maxx = 0, maxy = 0;
        startText = calcAlignment();
        for(int i=0; i<components.length; i++) {
            Dimension dim = components[i].getPreferredSize();
            maxx = maxx < dim.width ? dim.width : maxx;
            maxy = maxy < dim.height ? dim.height : maxy;
        }
        Dimension d = new Dimension(maxx + inset.left + inset.right,
                             components.length * maxy + inset.top + inset.bottom);
        return d;
    }
	/**
	 * set the "commands" of this menu. most probably it should be a Vector
	 * of Hashtable for a multiple command menu. but it could be a single
	 * Hashtable
	 */
    public void setCommands(Object plist) {
        if(plist instanceof Hashtable)
            setCommands((Hashtable) plist);
        else
            setCommands((Vector) plist);
    }
	/** a single command menu */
    public void setCommands(Hashtable h) {
        Vector v = new Vector();
        v.addElement(h);
        setCommands(v);
    }
	/** a multiple command menu */
    public void setCommands(Vector v) {
        startText = -1;
        int i,n = v.size();
        components = new MenuCell[n];
        for(i=0; i<n; i++)
            components[i] = new MenuCell(this, (Hashtable) v.elementAt(i));
        commands = v;
    }

	/**
	 * return the submenu window, if any. it is unlikely as submenu windwo
	 * exist only when displayed.
	 */
    public MenuWindow submenu() { return submenu; }
    Object getChild(Hashtable h) {
        Object o = h.get("child");
        if(o != null) {
            if(o instanceof String) { // load vector from URL
                try {
                    URL url = null;
					if(controller.applet != null)
						url = new URL(controller.applet.getDocumentBase(), (String) o);
					else
	                    url = URLBuilder.find((String) o);
                    if(url == null)
                        throw new NullPointerException(o + " not found");
                    o = PList.read(new InputStreamReader(url.openStream()));
                    if(o instanceof Vector)
                        h.put("child", o);
                    else
                        throw new ClassCastException("child's referred URL must point to a Vector PList");
                }
                catch (Exception ex) {
                    System.err.println(ex);
                    ex.printStackTrace();

                    Hashtable h2 = new Hashtable();
                    h2.put("name", "invalid URL: " + o);
                    Vector v = new Vector();
                    v.addElement(h2);
                    o = v;
                    h.put("child", v);
                }
            }
            else if(o instanceof Vector);
            else throw new IllegalArgumentException("child must be either an URL or a Vector of commands");
        }
        return o;
    }
    void showSubmenu(int s) {
		if(s == selected && submenu != null)
			return;

        if(submenu != null) {
            submenu.menu().setSelected(-1);
            submenu.setVisible(false);
            submenu = null;

			// work around a java applet bug which is
			// setVisible(false) on some windows could toBack() other
			if(controller.applet != null && s == -1)
				controller.todo.add(new ShowAction(controller.root));

			// work around a java applet bug which is
			// show() cancel repaint
			controller.todo.add(new RepaintAction(this));
        }
        if(s != -1 && components[s].isDir()) {
            submenu = new MenuWindow();

            Hashtable cmd = (Hashtable) commands.elementAt(s);
            submenu.configure(this, getChild(cmd), s);
            submenu.show();
            submenu.toFront();
        }
    }
	/**
	 * set the selected menu item. eventually display its submenu, if any.
	 */
    public void setSelected(int s) {
		if(s < -1 || s >= components.length)
			s = -1;

		if(controller.applet != null)
			controller.applet.showStatus(s == -1
				? (String) null
				: (String) components[s].getCommands().get("statut"));

		boolean invalid = false;
		if(s != selected) {
			invalid = true;

			if(selected != -1)
				components[selected].setSelected(false);
			if(s != -1)
				components[s].setSelected(true);
		}

		synchronized(controller) {
			showSubmenu(s);
			selected = s;
		}

		if(invalid)
			controller.todo.add(new RepaintAction(this));
    }
	void toFront() {
		// recursively make toFront() on each submenu
		// work around some bug
		if(submenu != null) {
			submenu.toFront();
			submenu.menu().toFront();
		}
	}

    public Rectangle getCellRect(int i) {
        if(i < 0)
            return new Rectangle(0, 0, -1, -1);
        Dimension d = components[i].getSize();
        return new Rectangle(inset.left, inset.top + i * d.height, d.width, d.height);
    }

    protected int calcAlignment() {
        if(components == null || components.length == 0)
            return 0;
        int x0 = 0, l;
        for(int i=0; i<components.length; i++) {
            l = components[i].getStartTextPosition();
            if(l > x0)
                x0 = l;
        }
        return x0;
    }
    public int getStartTextPosition() { return startText; }

	// double-buffering, avoid flickering
    protected Graphics getBufGraphics() {
        Dimension size = getSize();

        if(back_buffer == null)
            back_buffer = createImage(size.width, size.height);
        if(back_buffer.getWidth(null) != size.width
           || back_buffer.getHeight(null) != size.height)
            back_buffer = createImage(size.width, size.height);

        Graphics g = back_buffer.getGraphics();
        g.setClip(0, 0, size.width, size.height);

        return g;
    }
    public void update(Graphics g) { paint(g); }
    public void paint(Graphics g)
    {
        if(startText == -1)
            startText = calcAlignment();
        Graphics g2 = getBufGraphics();

        paintBackground(g2);
        g2 = paintBorder(g2);
        for(int i=0; i<components.length; i++)
            paintCells(g2, i);

        g.drawImage(back_buffer, 0, 0, null);
    }
    public void paintBackground(Graphics g) {
        Dimension d = getSize();
        if(controller.bgimage != null) {
            int w = controller.bgimage.getWidth(null);
            int h = controller.bgimage.getHeight(null);
            for(int i=0; i<d.width; i+=w)
                for(int j=0; j<d.height; j+=h)
                    g.drawImage(controller.bgimage, i, j, null);
        }
        else {
            Color c = g.getColor();
            g.setColor(getBackground());
            g.fillRect(0, 0, d.width, d.height);
            g.setColor(c);
        }
    }
    public Graphics paintBorder(Graphics g) {
        Dimension d = getSize();
        Color c = g.getColor();
        g.setColor(Color.black);
        g.drawRect(0, 0, d.width-1, d.height-1);
        g.setColor(c);
        return g.create(1, 1, d.width-2, d.height-2);
    }
    public Point getCellPosition(int i) {
        Dimension dim = getSize();
        Dimension cell = components[i].getSize();
        Point p = new Point(0, i * cell.height);
        return p;
    }
    public void paintCells(Graphics g, int i) {
        Rectangle r = g.getClipBounds();
        Dimension d = components[i].getSize();
        Point p = getCellPosition(i);
        if(r!=null && !r.intersects(new Rectangle(p.x, p.y, d.width, d.height)))
            return;
        g = g.create(p.x, p.y, d.width, d.height);
        components[i].setSelected(selected == i);
        components[i].paint(g);
        g.dispose();
    }
    protected void processComponentEvent(ComponentEvent ce)
    {
        if(components != null && components.length != 0) {
            Dimension dim = getSize();
			Dimension pref = getPreferredSize();
			if(pref.height <  dim.height)
	            dim = new Dimension(dim.width - inset.left - inset.right,
                                    (pref.height - inset.top - inset.bottom) / components.length);
			else
	            dim = new Dimension(dim.width - inset.left - inset.right,
                                    (dim.height - inset.top - inset.bottom) / components.length);
            for(int i=0; i<components.length; i++)
                components[i].setSize(dim);
        }
        super.processComponentEvent(ce);
    }
    protected void processMouse(MouseEvent me) {
        int ligne = -1;

        if(components != null && components.length !=0) {
            Dimension dim = getSize();
            dim = new Dimension(dim.width-2, dim.height-2);
            int x = me.getX() - inset.left;
            int y = me.getY() - inset.top;
            if(x >= 0 && x < dim.width && y >= 0 && y < dim.height)
                ligne = y / components[0].getSize().height;

            if(ligne >= components.length)
                ligne = -1;
        }

		// post selection.
		// so a nervous, or not enough precise, user wouldn't be bored.
        switch(me.getID()) {
            case MouseEvent.MOUSE_RELEASED:
                if(ligne == selected
					&& ligne != -1
					&& components[ligne].click()) {
						controller.handler.test();
						controller.root.setSelected(-1);
					}
				break;
            case MouseEvent.MOUSE_MOVED:
            case MouseEvent.MOUSE_ENTERED:
				if(ligne != selected) {
					if(selected == -1) {
						controller.handler.test();
						setSelected(ligne);
					}
					else {
						int delay = 100;
						if(ligne == -1)
							delay = 600;
						Action a = new ActionSelectMenu(this, ligne);
                        controller.handler.test(a, delay);
					}
				}
                else {
					controller.handler.test();
					setSelected(ligne);
					toFront();
				}
                break;
            case MouseEvent.MOUSE_EXITED:
				Action a = new ActionSelectMenu(controller.root, -1);
				controller.handler.test(a, 600);
				break;
        }
    }
	// group mouse event
    protected void processMouseMotionEvent(MouseEvent me) {
        super.processMouseMotionEvent(me);
        processMouse(me);
    }
	// group mouse event
    protected void processMouseEvent(MouseEvent me) {
        super.processMouseEvent(me);
        processMouse(me);
    }

	protected void finalize()
		throws Throwable
	{
		setSelected(-1);
		super.finalize();
	}
}